[% SET mnt_pfx = c.mount_url %]

[% SET
  foundation_path = mnt_pfx _ '/assets/rapidapp/misc/static/login/foundation/'
%]

<!DOCTYPE html>
<!--[if lt IE 8]> <html class="lt-ie8" lang="en"> <![endif]-->
<!--[if gte IE 8]><!--> <html lang="en"> <!--<![endif]-->
<head>
  <meta charset="utf-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0" />

  [% c.all_html_head_tags %]

  <title>[% title %]</title>

  <!-- Foundation framework -->
  <link rel="stylesheet" href="[% foundation_path %]stylesheets/foundation.css">
  <!--[if lt IE 8]> <link rel="stylesheet" href="[% foundation_path %]stylesheets/ie.css" /> <![endif]-->
  <link rel="stylesheet" href="[% mnt_pfx %]/assets/rapidapp/misc/static/login/css/style.css">
</head>

<body class="ptrn_a grdnt_a">

  <div style="padding-top:60px;"></div>
  <div class="container">
    <div class="row">
      <div class="eight columns centered" style="padding-top:0px;max-width:520px;">
        <div class="login_box" style="max-width:500px;">
          
          <div class="login_logo" style="top:-115px;">
            [% IF login_logo_url %]
              <img src="[% login_logo_url %]" alt="" />
            [% END %]
          </div>
          
          <div class="lb_content">
            <div class="cf" style="height:80px;">
              <h2 class="lb_ribbon lb_blue">
                <div style="padding-top:3px;">
                  <img src="[% mnt_pfx %]/assets/rapidapp/misc/static/images/iconLoggedOut.png" style="vertical-align:middle;padding-bottom:5px;padding-right:9px;">
                  <span style="padding-right:11px;font-weight:bold;font-size:1.1em;">Sign in</span>
                </div>
              </h2>

              <!--
              <a href="/password_reset" class="right small sl_link">
                <span>Password Reset</span>
              </a>
              
              <a href="/obtain_username" class="right small sl_link">
                <span>Obtain Username</span>
              </a>
              -->
              
              <!--
              <div class="right">
                <span>[% ver_string %]</span>
              </div>
              -->
            </div>

            [% IF error_status %]
              <div style="font-size:1.1em;font-weight:bold;color:red;padding:10px;">
                <center>&nbsp;[% error_status %]&nbsp;</center>
              </div>
            [% END %]
            
            <div class="row m_cont">
              <div class="eight columns centered">
                <div class="l_pane">
                  [% INCLUDE rapidapp/public/login_form.tt %]
                </div>
                
                <!--
                <div class="l_pane" style="display:none">
                  <form action="index.php" method="post" class="nice" id="rp_form">
                    <div class="sepH_c">
                      <p class="sepH_b">Please enter your email address. You will receive a link to create a new password via email.</p>
                      <div class="elVal">
                        <label for="upname">E-mail:</label>
                        <input type="text" id="upname" name="upname" class="oversize expand input-text" />
                      </div>
                    </div>
                    <div class="cf">
                      <input type="submit" class="button small radius right black" value="Get new password" />
                    </div>
                  </form>
                </div>
                -->
              </div>
            </div>
          </div>
        </div>

        <div style="position: absolute; bottom: 0px; width:100%;">
          <div style="text-align:center;padding-bottom:5px;height:100%;">
            <a target="_blank" href="http://www.rapidapp.info/">
              <img 
                width="165" height="25" border="0" 
                src="[% mnt_pfx %]/assets/rapidapp/misc/static/images/rapidapp_powered_logo_small.png"
              >
            </a>
          </div> 
        </div>
        
        <div style="position: absolute; bottom: -18px; width:100%;">
          <div class="right" style="float:right;padding-right:15px;font-size:.7em;color:#666666;">
            <span>[% ver_string %]</span>
          </div>
        </div>
        
      </div>
    </div>
  </div>
  
</body>
</html>
