/* xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
xx  CSS Table Gallery               xx
xx  Author: Stu Nicholls            xx
xx  URL: http://www.cssplay.co.uk/  xx
xx  Country: England                xx
xx  Date: 2005-10-23                xx
xx  xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx */
table.tabular-table {
  border-collapse:collapse !important; 
  font-family:verdana, arial, sans-serif !important; 
/*  font-size:0.75em !important;  */
  line-height:2em !important; 
  border:1px solid #363 !important;
  }
td.tabular-table, th.tabular-table {
  padding:0 0.5em !important; 
  border-top:1px solid #ddd !important;
  }
tr.tabular-table.odd {
  background:#eee !important;
  }
th.tabular-table {
  background:#839965 !important;
  }
caption.tabular-table, tfoot.tabular-table {
  font-family:georgia, "times new roman", serif !important; 
  font-size:3em !important; 
  line-height:2em !important; 
  height:2em !important; 
  padding-right:0.25em !important; 
  text-align:right !important; 
  color:#363 !important; 
  border:1px solid #ddd !important;
  }
tfoot.tabular-table th.tabular-table {
  background:#fff !important; 
  color:#363 !important; 
  font-weight:normal !important;
  }
thead.tabular-table tr.tabular-table {
  color:#fff !important; 
  height:4em !important;
  }
thead.tabular-table tr.tabular-table th.tabular-table:first-child {
  color:#fc6 !important; 
  border-right:1px solid #000 !important; 
  text-align:right !important; 
  padding-right:1.5em !important;
  }
tbody.tabular-table th.tabular-table {
  padding:0 !important; 
  border-right:1px solid #fff !important; 
  border-top:1px solid #000 !important;
  width:16em !important; 
  vertical-align:middle !important;
  }
tbody.tabular-table tr.tabular-table.odd th.tabular-table {
  border-right:1px solid #eee !important;
  }
tbody.tabular-table td.tabular-table {
  color:#363 !important; 
  vertical-align:middle !important; 
  height:4em !important;
  }
tbody.tabular-table th.tabular-table a.tabular-table {
  color:#fc3 !important; 
  text-decoration:none !important; 
  border-right:0.8em solid #fc3 !important; 
  font-weight:normal !important; 
  text-align:right !important; 
  padding-right:0.5em !important; 
  float:right !important; 
  border-top:0.8em solid #839965 !important; 
  border-bottom:0.8em solid #839965 !important;
  }
tbody.tabular-table tr.tabular-table th.tabular-table a.tabular-table:visited {
  border-right:0.8em solid #fff !important;
  }
tbody.tabular-table tr.tabular-table.odd th.tabular-table a.tabular-table:visited {
  border-right:0.8em solid #eee !important;
  }
tbody.tabular-table td.tabular-table a.tabular-table:visited {
  color:#000 !important; 
  font-weight:normal !important; 
  text-decoration:line-through !important;
  }
tbody.tabular-table td.tabular-table a.tabular-table {
  color:#000 !important; 
  font-weight:normal !important;
  }
tbody.tabular-table tr.tabular-table th.tabular-table a.tabular-table:hover {
  color:#fff !important; 
  border-right:0.8em solid #fff !important;
  }
tbody.tabular-table tr.tabular-table.odd th.tabular-table a.tabular-table:hover {
  color:#fff !important; 
  border-right:0.8em solid #eee !important;
  }
tbody.tabular-table td.tabular-table a.tabular-table:hover {
  color:#c00 !important; 
  font-weight:normal !important; 
  text-decoration:none !important;
  }


.tabular-table-row-head {
  color:#fc3 !important; 
  text-decoration:none !important; 
  border-right:0.8em solid #fc3 !important; 
  font-weight:normal !important; 
  text-align:right !important; 
  padding-right:0.5em !important; 
  float:right !important; 
  border-top:0.8em solid #839965 !important; 
  border-bottom:0.8em solid #839965 !important;
  }

}

